@@ -5,35 +5,29 @@ class attributes by using ``Selector``:s. There are several selectors provided
 in this module to cover all regular cases of data acquisition.
 """
 #=======================================================================
+from itertools import chain
+from types import ModuleType
+import warnings
 from rdflib import RDF, RDFS, Namespace, URIRef, BNode, Literal
 from rdflib import ConjunctiveGraph
-from types import ModuleType
 try: import simplejson
 except ImportError: simplejson = None
 from oort.util.code import contract
 #=======================================================================
 
 
+# TODO: deprecate THIS_QUERY and then remove this?
 THIS_QUERY = object()
-DEFAULT_MAX_DEPTH = 20
+
+MODULE_SEP = ':'
 
 
-# TODO: optimization by providing an "execution context" and avoiding multiple queries for the same uri etc.
-# FIXME: complete; pass along execContext (and maxDepth?)
-def run_queries(queries, graph, subject, lang):
-    execContext = {}
-    for query in queries:
-        key = (id(query), str(subject))
-        if key not in execContext:
-            execContext[key] = query(graph, subject, lang)
-        yield execContext[key]
-
-
-#=======================================
+#-----------------------------------------------------------------------
 
 
 class Selector(object):
-    __slots__ = ('predicate', '_namespace', 'subQuery', 'filters',
+    __slots__ = ('predicate', '_namespace', 'filters',
+                 '_subQueryMarker', '_finalSubQuery',
                  '_name', '_queryClass')
 
     def __init__(self, predBase=None, subQuery=None):
@@ -43,15 +37,14 @@ class Selector(object):
             self._namespace = predBase
         elif isinstance(predBase, URIRef):
             self.predicate = predBase
-        self.subQuery = subQuery
+        self._subQueryMarker = subQuery
+        self._finalSubQuery = False
         self.filters = []
 
     @contract.state_change
     def hook_into_rdf_query(self, name, queryClass):
         self._name = name
         self._queryClass = queryClass
-        if self.subQuery is THIS_QUERY:
-            self.subQuery = self._queryClass
         if not self._namespace:
             self._namespace = queryClass._namespace
         if not self.predicate and self._namespace:
@@ -64,10 +57,28 @@ class Selector(object):
         #    raise ValueError(
         #            "Could not determine predicate for Selector %s" % self)
 
-    def __get__(self, parentQuery, _parentQueryClass=None):
-        prep = parentQuery._preparedSelects[self._name]
+    def get_sub_query(self):
+        final = self._finalSubQuery
+        if final is False:
+            marker = self._subQueryMarker
+            final = marker
+            if isinstance(marker, basestring):
+                if MODULE_SEP in marker:
+                    module, marker = marker.split(MODULE_SEP)
+                else:
+                    module = self._queryClass.__module__
+                final = __import__(module, fromlist=['']).__dict__[marker]
+            elif marker is THIS_QUERY:
+                final = self._queryClass
+            self._finalSubQuery = final
+        return final
+
+    def __get__(self, rdfQueryInstance, rdfQueryOwnerClass=None):
+        if not rdfQueryInstance:
+            return self
+        prep = rdfQueryInstance._preparedSelects[self._name]
         if not prep.hasRun:
-            result = self.retreive_result(parentQuery, prep.selectArgs)
+            result = self.retreive_result(rdfQueryInstance, prep.selectArgs)
             for fltr in self.filters:
                 result = fltr(result)
             prep.hasRun = True
@@ -75,31 +86,32 @@ class Selector(object):
         return prep.result
 
     @contract.default_method
-    def retreive_result(self, parentQuery, selectArgs):
+    def retreive_result(self, rdfQueryInstance, selectArgs):
         result = self._process_for_subqueries(
+                rdfQueryInstance,
                 self.select(*selectArgs),
                 *selectArgs[:2]
             )
         return result
 
-    # TODO: if subQuery is None but resource isn't a literal, use rdf:type to
-    # find a proper RdfQuery (by looking in a register of RdfQuery subclasses
-    # with a declared SCHEMA)? Or is type_switch enough?
-    def _process_for_subqueries(self, rawresults, graph, lang):
+    def _process_for_subqueries(self, rdfQueryInstance,
+            rawresults, graph, lang):
         returnList = isinstance(rawresults, list)
         if not rawresults:
             if returnList: return []
             else: return None
 
-        subQuery = self.subQuery
+        subQuery = self.get_sub_query()
         if not subQuery:
             return rawresults
         else:
-            # FIXME: using THIS_QUERY may currently cause infinite loops.
+            # TODO: using THIS_QUERY (and no execCache?) may currently cause
+            # infinite loops..? But things are more lazy now; perhaps not..
+            run_query = query_or_cached(subQuery, rdfQueryInstance._execCache)
             if returnList:
-                return [subQuery(graph, lang, uri) for uri in rawresults]
+                return [run_query(graph, lang, uri) for uri in rawresults]
             else:
-                return subQuery(graph, lang, rawresults)
+                return run_query(graph, lang, rawresults)
 
     @contract.template_method
     def select(self, graph, lang, subject):
@@ -110,18 +122,24 @@ class Selector(object):
     def back_to_graph(self, graph, subject, value):
         pass
 
-    def __rshift__(self, subQuery):
-        self.subQuery = subQuery
+    def viewed_as(self, subQuery):
+        self._subQueryMarker = subQuery
         return self
 
+    def __rshift__(self, subQuery):
+        return self.viewed_as(subQuery)
+
+    def add_filter(self, fltr):
+        self.filters.append(fltr)
+
     def __or__(self, fltr):
-        self.filters.append(fltr)
+        self.add_filter(fltr)
         return self
 
-    def __set__(self, parentQuery, value):
-        prep = parentQuery._preparedSelects[self._name]
-        lang = parentQuery._lang
-        sub = self.subQuery
+    def __set__(self, rdfQueryInstance, value):
+        prep = rdfQueryInstance._preparedSelects[self._name]
+        lang = rdfQueryInstance._lang
+        sub = self.get_sub_query()
         if isinstance(value, list):
             value = [self.type_raw_value(val, lang) for val in value]
         else:
@@ -139,7 +157,7 @@ class Selector(object):
     @contract.default_method
     def type_raw_value(self, value, lang):
         # TODO: need more clever type mapping. also, allow {'_uri': ".." } to be resource?
-        if isinstance(value, str.__base__):
+        if isinstance(value, basestring):
             value = Literal(value)
         return value
 
@@ -154,7 +172,7 @@ class PreparedSelect(object):
 
 class _rdf_query_meta(type):
     def __init__(cls, clsName, bases, clsDict):
-        type.__init__(cls, clsName, bases, clsDict)
+        super(_rdf_query_meta, cls).__init__(clsName, bases, clsDict)
 
         cls._selectors = selectors = {}
         for base in bases:
@@ -185,21 +203,29 @@ class RdfQuery(object):
 
     RDF_TYPE = RDFS.Resource
 
-    def __init__(self, graph, lang, subject, depth=DEFAULT_MAX_DEPTH):
-        # TODO: pass depth and use to avoid infinite recursion
-        if depth == 0:
-            raise NotImplementedError("TODO: max depth exception..")
+    # TODO: test use of execCache propertly (it seems to work though)
 
+    def __init__(self, graph, lang, subject, execCache=None):
         self._graph = graph
         self._subject = subject
         self._lang = lang
-        self._preparedSelects = prepareds = {}
+        self._preparedSelects = self._make_prepare_selects()
+        self._execCache = execCache
+
+    def _make_prepare_selects(self):
+        prepareds = {}
+        graph, lang, subject = self._graph, self._lang, self._subject
         for name, selector in self._selectors.items():
-            if not subject: # FIXME: happens when subject is a bnode?
-                setattr(self, name, None)
+            if not subject:
+                # FIXME: happens when subject is a string/Literal - wrong in 
+                # itself! Remove or signal error? As it is, it leads to 
+                # illegible errors further down!
+                # Also, why not: if subject == u'':
+                #setattr(self, name, None) # TODO:removed; see this fixme
                 continue
             prep = PreparedSelect(graph, lang, subject)
             prepareds[name] = prep
+        return prepareds
 
     def __str__(self):
         return str(self._subject)
@@ -210,13 +236,6 @@ class RdfQuery(object):
         else:
             return self._subject == other
 
-    @property
-    def uri(self):
-        return self._subject
-
-    def get_selected_value(self, name):
-        return self._preparedSelects[name].result
-
     @classmethod
     def bound_with(cls, subject, lang=None):
         def bound_query(graph, _lang, _subject):
@@ -225,6 +244,29 @@ class RdfQuery(object):
         bound_query.__name__ = cls.__name__
         return bound_query
 
+    @classmethod
+    def from_dict(cls, data, lang, subject):
+        graph = ConjunctiveGraph()
+        query = cls(graph, lang, subject)
+        for k, v in data.items():
+            setattr(query, k, v)
+        return query
+
+    @classmethod
+    def find_by(cls, graph, lang, execCache=None, **kwargs):
+        assert len(kwargs) == 1
+        name, value = kwargs.items()[0]
+        predicate = cls._selectors[name].predicate
+        for subject in graph.subjects(predicate, value):
+            yield query_or_cached(cls, execCache)(graph, lang, subject)
+
+    @property
+    def uri(self):
+        return self._subject
+
+    def get_selected_value(self, name):
+        return self._preparedSelects[name].result
+
     def to_graph(self, lgraph=None):
         subject = self._subject or BNode() # FIXME: is this ok?
         if not subject: return # FIXME, see fixme in __init__
@@ -285,13 +327,50 @@ class RdfQuery(object):
         else:
             raise NotImplementedError
 
-    @classmethod
-    def from_dict(cls, data, lang, subject):
-        graph = ConjunctiveGraph()
-        query = cls(graph, lang, subject)
-        for k, v in data.items():
-            setattr(query, k, v)
-        return query
+
+#-----------------------------------------------------------------------
+
+
+# Is the use of weakref fine enough (reasonably needed to avoid cyclic
+# references and hence possible memory leaks)?
+# See: <http://docs.python.org/lib/module-weakref.html>
+from weakref import WeakValueDictionary
+
+class ExecCache(object):
+    """
+    This is a query execution cache which reuses results for the same query,
+    subject and language, avoiding multiple instances of the same query when
+    given the same subject and lang.
+    """
+    def __init__(self):
+        self.cache = WeakValueDictionary()
+    def __call__(self, query, graph, lang, subject):
+        cache = self.cache
+        key = (id(query), unicode(subject), lang)
+        #key = (query, subject, lang)
+        result = cache.get(key)
+        if not result:
+            result = query(graph, lang, subject, self)
+            cache[key] = result
+        return result
+
+
+def query_or_cached(rdfQuery, execCache):
+    if execCache:
+        def run_query(graph, lang, uri):
+            return execCache(rdfQuery, graph, lang, uri)
+        return run_query
+    else:
+        return rdfQuery
+
+
+def run_queries(queries, graph, lang, subject):
+    execCache = ExecCache()
+    for query in queries:
+        yield execCache(query, graph, lang, subject)
+
+
+#-----------------------------------------------------------------------
 
 
 class Filter(object):
@@ -302,8 +381,13 @@ class Filter(object):
 
 
 class Sorter(object):
-    def __init__(self, attr=None, reverse=False, ignoreCase=False):
-        self.attr = attr
+    def __init__(self, obj=None, reverse=False, ignoreCase=False):
+        if callable(obj):
+            self.attr = None
+            self.func = obj
+        else:
+            self.attr = obj
+            self.func = None
         self.reverse = reverse
         self.ignoreCase = ignoreCase
     def __call__(self, items):
@@ -314,8 +398,11 @@ class Sorter(object):
         return copy
     def sort(self, r1, r2):
         attr = self.attr
+        func = self.func
         if attr:
-            v1, v2 = getattr(r1, attr), getattr(r2, attr)
+            v1, v2 = getattr(r1, attr, r1), getattr(r2, attr, r2)
+        elif func:
+            v1, v2 = func(r1), func(r2)
         else:
             v1, v2 = r1, r2
         if self.ignoreCase:
@@ -323,14 +410,21 @@ class Sorter(object):
         return cmp(v1, v2)
 
 
-#=======================================
+#-----------------------------------------------------------------------
 
 
 # TODO: totally untested!
+#   - use: TypeSwitch(persons=Person, values=Literal)
+#   - a subclass of RdfQuery? Or affect selector..? Reasonably yes..
+#   - should adapt to if stuff is a list or one thing (one or each)
+#       - how about localized?
+#   - also should be used as list *or*:
+#       - obj.switchedstuff.persons
 #def type_switch(typeSelectors, default):
 #    rdfType = graph.value(resource, RDF.type, None, any=True)
-#    def select(graph, lang, resource):
-#        return typeSelectors.get(rdfType, default)
+#    def select(graph, lang, resource, **kwargs):
+#        query = typeSelectors.get(rdfType, default)
+#        return query(graph, lang, resource, **kwargs)
 #    return select
 
 
@@ -382,8 +476,8 @@ class each_where_self_is(Selector):
 
 
 class collection(Selector):
-    def __init__(self, base, subQuery=None, multiple=False):
-        Selector.__init__(self, base, subQuery)
+    def __init__(self, predBase=None, subQuery=None, multiple=False):
+        Selector.__init__(self, predBase, subQuery)
         self.multiple = multiple
 
     def select(self, graph, lang, subject):
@@ -412,7 +506,7 @@ class collection(Selector):
 
 class TypeLocalized(Selector):
     def type_raw_value(self, value, lang):
-        if isinstance(value, str.__base__):
+        if isinstance(value, basestring):
             value = Literal(value, lang)
         return value
 
@@ -427,23 +521,28 @@ class localized(TypeLocalized, UnarySelector):
 
 
 # TODO: This is a hackish solution; see also below (transparently using datatype).
-#       It also reduces the literal, making it irreversible.
-from oort.util._genshifilters import language_filtered_xml
-class localized_xml(UnarySelector):
-    """This selector removes any elements with an xml:lang other than the 
-    current language. It also supports the never standardized 'rdf-wrapper' in 
-    XML Literals, who are removed from the output.
+#       It also reduces the literal, making it irreversible (should store original value!).
+try:
+    from oort.util._genshifilters import language_filtered_xml
+except ImportError, e:
+    warnings.warn("Could not import _genshifilters. Error was: %r. The selector 'localized_xml' will not be available." % e)
+else:
+    class localized_xml(UnarySelector):
+        """This selector removes any elements with an xml:lang other than the 
+        current language. It also supports the never standardized 'rdf-wrapper' in 
+        XML Literals, who are removed from the output.
 
-    Important! This is currently tied to the Genshi Templating System, and may 
-    not work as expected in all cases."""
-    def select(self, graph, lang, subject):
-        return language_filtered_xml(
-                graph.objects(subject, self.predicate), lang)
+        Important! This is currently tied to the Genshi Templating System, and may 
+        not work as expected in all cases."""
 
-    def type_raw_value(self, value, lang):
-        if isinstance(value, str.__base__):
-            value = Literal(value, datatype=RDF.XMLLiteral)
-        return value
+        def select(self, graph, lang, subject):
+            return language_filtered_xml(
+                    graph.objects(subject, self.predicate), lang)
+
+        def type_raw_value(self, value, lang):
+            if isinstance(value, basestring):
+                value = Literal(value, datatype=RDF.XMLLiteral)
+            return value
 
 class i18n_dict(Selector):
     def select(self, graph, lang, subject):
@@ -463,19 +562,37 @@ class each_localized(TypeLocalized, EachSelector):
                  if value.language == lang ]
 
 
-#=======================================
+#-----------------------------------------------------------------------
 
 
+# TODO: Though "widely used" (by me), I think this was a little premature.
+# There seems little use for this that a regular property can't do (getting the
+# graph, lang and subject from self -- where needed). Perhaps I should include
+# a memoized codeutil so it's easy to create lazily calculated bigger things.
+#
+# Even worse, this "utility" bypasses _process_for_subqueries, which is
+# intricate and very close to the implementation. And this is the only reason
+# retreive_result is marked as a "default_method"; it should reasonably be
+# private.
+#
 class selector(Selector):
     "Use as decorator for methods of an RdfQuery subclass to convert them to selectors."
     def __init__(self, func):
         super(selector, self).__init__(None)
         self.func = func
-    def retreive_result(self, parentQuery, selectArgs):
-        return self.func(parentQuery, *selectArgs)
+    def retreive_result(self, rdfQueryInstance, selectArgs):
+        return self.func(rdfQueryInstance, *selectArgs)
+    @classmethod
+    def filtered_by(cls, *filters):
+        def decorator(func):
+            sel = cls(func)
+            for fltr in filters:
+                sel.add_filter(fltr)
+            return sel
+        return decorator
 
 
-#=======================================
+#-----------------------------------------------------------------------
 # TODO: consider returning ElementTree data for XML Literals. And if so, also
 # filtered on u'{http://www.w3.org/XML/1998/namespace}lang' (keep if none) for localized.
 
@@ -483,12 +600,85 @@ class selector(Selector):
 # Use: rdflib.Literal.castPythonToLiteral
 # See: rdflib.sparql.sparqlOperators.getLiteralValue(v)
 # See: <http://en.wikipedia.org/wiki/RDFLib#RDF_Literal_Support>
+# - NOTE: Isn't this done automatically by rdflib now? I believe so.
 
 
-#=======================================
+#-----------------------------------------------------------------------
 
-def chain(*iters):
-    for iterator in iters:
-        for i in iterator: yield i
+
+class QueryContext(object):
+    """
+    A query context, used to provide a managed context for query execution.
+
+    Initalized with:
+
+    -   graph
+    -   language or getter for language
+    -   a set of queries or a modules containing queries
+        Accessible as attributes on the context or via view_for using RDF_TYPE
+
+    """
+
+    def __init__(self, graph, langobj, queries=None, query_modules=None):
+        self._graph = graph
+        self._execCache = ExecCache()
+
+        if callable(langobj):
+            get_lang = langobj
+        else:
+            def get_lang(): return langobj
+        self._get_lang = get_lang
+
+        self._querydict = querydict = {}
+        if queries:
+            for query in queries:
+                querydict[query.__name__] = query
+        if query_modules:
+            for module in query_modules:
+                for name, obj in module.__dict__.items():
+                    if isinstance(obj, type) and issubclass(obj, RdfQuery):
+                        querydict[name] = obj
+
+        self._queryTypeMap = {}
+        for query in querydict.values():
+            self._queryTypeMap[query.RDF_TYPE] = query
+
+    def __getattr__(self, name):
+        try:
+            query = self._querydict[name]
+            return self._prepared_query(query)
+        except KeyError:
+            raise AttributeError("%s has no attribute '%s'" % (self, name))
+
+    def view_for(self, uriref):
+        for typeref in self._graph.objects(uriref, RDF.type):
+            query = self._queryTypeMap.get(typeref)
+            if query:
+                return self._prepared_query(query)(uriref)
+        raise KeyError("%s has no query for type '%s'" % (self, uriref))
+
+    def _prepared_query(self, query):
+        return self.PreparedQuery(self, query)
+
+    class PreparedQuery(object):
+        __slots__ = ('query', 'context')
+
+        def __init__(self, context, query):
+            self.context = context
+            self.query = query
+
+        def __call__(self, subject):
+            cx = self.context
+            return cx._execCache(self.query, cx._graph, cx._get_lang(), subject)
+
+        def find_all(self):
+            cx = self.context
+            for subject in cx._graph.subjects(RDF.type, self.query.RDF_TYPE):
+                yield cx.view_for(subject)
+
+        def find_by(self, **kwargs):
+            cx = self.context
+            return self.query.find_by(cx._graph, cx._get_lang(),
+                    execCache=cx._execCache, **kwargs)
 
 
