@@ -2,8 +2,14 @@
 
 
 class autosuper(type):
-    def __init__(cls, name, bases, dict):
-        super(autosuper, cls).__init__(name, bases, dict)
+    """
+    Use this as a metaclass for types to set the magic attribute '__super' on
+    them. When this is accessed through self the result is an object
+    constructed as if super(ThisType, self) had been called. (Makes it a little
+    bit more like in Python 3, where super() will work like this attribute.)
+    """
+    def __init__(cls, name, bases, members):
+        super(autosuper, cls).__init__(name, bases, members)
         setattr(cls, "_%s__super" % name, super(cls))
 
 
@@ -17,6 +23,28 @@ def init_slots(obj, *args, **kwargs):
         setattr(obj, a, data.get(a, None))
 
 class SlotStruct(object):
+    """
+    Base class for making "struct" types with members defined by their
+    __slots__ value. Example:
+
+        >>> class Item(SlotStruct):
+        ...     __slots__ = 'first', 'last'
+
+        >>> item1 = Item(first='a', last='b')
+        >>> item1.first, item1.last
+        ('a', 'b')
+
+        >>> item1.middle = '-'
+        Traceback (most recent call last):
+        ...
+        AttributeError: 'Item' object has no attribute 'middle'
+
+        >>> item2 = Item('a', 'b')
+        >>> assert (item1.first, item1.last) == (item2.first, item2.last)
+
+        >>> assert item1.make_dict() == {'first': 'a', 'last': 'b'}
+
+    """
     __slots__ = ()
     def __init__(self, *args, **kwargs):
         init_slots(self, *args, **kwargs)
