@@ -1,36 +1,40 @@
 #=======================================================================
 import os
 from os.path import splitext, expanduser
-from rdflib import Namespace
+import logging
+from rdflib import Namespace, Literal
 #=======================================================================
-# TODO: (optional?) poorMansInference!
-#           - interpret rdfs (also using equiv. owl terms)
-#           - optional vocabulary discovery?
+# TODO: optional hook for inference -- with e.g. FuXi?
+
+_logger = logging.getLogger(name=__name__)
 
 
 OUG_NS = Namespace('tag:oort.to,2006:system/util/graphs#')
 LAST_MOD = OUG_NS.lastmodified
 
+
 def load_if_modified(graph, fpath, format):
     dataUri = graph.absolutize(fpath)
     modTime = os.stat(fpath).st_mtime
     try:
         loadedModValue = graph.value(dataUri, LAST_MOD)
-        loadedModTime = int(loadedModValue)
+        loadedModTime = float(loadedModValue)
     except TypeError:
         loadedModTime = -1
     if modTime > loadedModTime:
         graph.remove((dataUri, LAST_MOD, loadedModValue))
         graph.remove_context(graph.context_id(dataUri))
-        print "Loading: %s" % dataUri
+        _logger.info("Loading: <%s>" % fpath)
         graph.load(fpath, publicID=dataUri, format=format)
-        graph.add((dataUri, LAST_MOD, modTime))
+        graph.add((dataUri, LAST_MOD, Literal(modTime)))
 
 
 DEFAULT_FORMAT_MAP = {
-        # TODO: rdf/a
         'rdf': 'xml',
-        'n3':'n3',
+        'n3': 'n3',
+        'ttl': 'n3',
+        'xhtml': 'rdfa',
+        'html': 'rdfa',
     }
 
 def get_format(fpath, fmap=None):
